'=============================================================================
' Copyright  Micromega Corporation 2005-2007. All rights reserved.
'
' @file		demo2-i2c.bas
' @target	PICBASIC PRO
'
' This program demonstrates the use of the uM-FPU V3 floating point
' coprocessor with the PICBASIC PRO compiler using an I2C interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point and displays them in degrees Celsius and degrees Fahrenheit.
' It also calculates the minimum and maximum temperatures.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	September 18, 2007
'	- modified for uM-FPU V3.1
'	V3.0 - October 23, 2006
'	- modified for uM-FPU V3
'   V2.0 - January 17, 2005
'   - modified for uM-FPU V2 and PICBASIC PRO compiler
'	V1.0 - January 8, 2005
'	- original version
'=============================================================================

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      ' specify the speed of the oscillator
OSC_SPEED   con 20                      ' define PICBASIC constant

'-------------------- debug definitions ---------------------------------------
define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  19200
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
Fpu_SCL     var     PORTC.3             ' I2C SCL pin (uM-FPU SCL)
Fpu_SDA     var     PORTC.4             ' I2C SDA pin (uM-FPU SDA)
Fpu_ID      con     $C8                 ' uM-FPU I2C device address
define      I2C_HOLD    1               ' required for correct I2C timing

include     "umfpuV3-i2c.bas"           ' include uM-FPU V3.1 support routines

'-------------------- DS1620 pin definitions ---------------------------------

DS_RST      var     PORTA.0             ' DS1620 reset/enable
DS_CLK      var     PORTA.1             ' DS1620 clock
DS_DATA     var     PORTA.2             ' DS1620 data

LSBFIRST	con		0					' shiftout mode
LSBPRE		con		1					' shiftin mode

'-------------------- uM-FPU register definitions ----------------------------

DegC            CON     1               ' temperature in degrees Celsius
DegC_Min        CON     2               ' minimum temperature
DegC_Max        CON     3               ' maximum temperature
DegF            CON     4               ' temperature in degrees Fahrenheit
F1_8            CON     5               ' constant 1.8

'-------------------- variables ----------------------------------------------

rawTemp         VAR     Word            ' raw temperature reading

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "Demo 2: "

  GOSUB Fpu_Reset                       ' initialize uM-FPU
  IF fpu_status <> SYNC_CHAR THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 ' display version string
  ENDIF
  DEBUG 13, 10, "---------------------"

  GOSUB Init_DS1620                     ' initialize DS1620

  DEBUG 13, 10, "               Min   Current   Max"
  DEBUG 13, 10, "              -----  -------  -----"

                                        ' load floating point constant
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, F1_8, ATOF, "1.8", 0, FSET0]

  GOSUB Read_Temp                       ' get initial temperature reading
                                        ' initialize min/max values
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, DegC_Min, FSET, DegC]
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, DegC_Max, FSET, DegC]


'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
  GOSUB Read_Temp                       ' get temperature reading
                                        ' calculate min/max values
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, DegC_Min, FMIN, DegC]
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, DegC_Max, FMAX, DegC]

  DEBUG 13, 10, "Celsius   "    		' display temperatures in Celsius
  reg = DegC_Min
  GOSUB Print_TempC
  reg = DegC
  GOSUB Print_TempC
  reg = DegC_Max
  GOSUB Print_TempC

  DEBUG 13, 10, "Fahrenheit"            ' display temperatures in Fahrenheit
  reg = DegC_Min
  GOSUB Print_TempF
  reg = DegC
  GOSUB Print_TempF
  reg = DegC_Max
  GOSUB Print_TempF
  DEBUG 13, 10

  PAUSE 2000                            ' delay, then get the next reading
  GOTO Main
  END

'-------------------- Read_Temp ----------------------------------------------

Read_Temp:
  GOSUB Read_DS1620                     ' read temperature value
                                        ' send rawTemp to uM-FPU
                                        ' convert to floating point
                                        ' store in register
                                        ' divide by 2 to get degrees Celsius
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0,_
	[SELECTA, DegC, LOADWORD, rawTemp, FSET0, FDIVI, 2]
  RETURN

'-------------------- Print_TempC --------------------------------------------
' input:  reg         selects the uM-FPU register to display
'-----------------------------------------------------------------------------

Print_TempC:
                                        ' select reg as register A
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0, [SELECTA, reg]

  format = 81                           ' print reg as formatted floating point
  GOSUB Print_FloatFormat
  RETURN

'-------------------- Print_TempF --------------------------------------------
' input:  reg         selects the uM-FPU register to convert and display
'-----------------------------------------------------------------------------

Print_TempF:
                                        ' degF = reg * 1.8 + 32
  I2CWRITE Fpu_SDA, Fpu_SCL, Fpu_ID, 0,_
	[SELECTA, DegF, FSET, reg, FMUL, F1_8, FADDI, 32]

  format = 81                           ' print reg as formatted floating point
  GOSUB Print_FloatFormat
  RETURN

'-------------------- Init_DS1620 --------------------------------------------

Init_DS1620:
  ADCON1 = 6                            ' configure A0-A3 for digital I/O
  LOW DS_RST                            ' initialize pin states
  HIGH DS_CLK
  PAUSE 100

  HIGH DS_RST                           ' configure for CPU control
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$0C, $02]
  LOW DS_RST
  PAUSE 100

  HIGH DS_RST                           ' start temperature conversions
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$EE]
  LOW DS_RST
  PAUSE 1000                            ' wait for first conversion
  RETURN

'-------------------- Read_DS1620 --------------------------------------------

Read_DS1620:
  HIGH DS_RST                           ' read temperature value
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$AA]
  SHIFTIN  DS_DATA, DS_CLK, LSBPRE, [rawTemp\9]
  LOW DS_RST                            ' extend the sign bit
  IF rawTemp.BIT8 = 1 THEN rawTemp.HIGHBYTE = $FF
  RETURN
