'=============================================================================
' Copyright  Micromega Corporation 2005-2007. All rights reserved.
'
' @file		demo1-spi.bas
' @target	PICBASIC PRO
'
' This program demonstrates the use of the uM-FPU V3.1 floating point
' coprocessor with the PICBASIC PRO compiler using a API interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point and displays them in degrees Celsius and degrees Fahrenheit.
'
' @author Cam Thompson, Micromega Corporation
' @version
'	September 18, 2007
'	- modified for uM-FPU V3.1
'	V3.0 - October 23, 2006
'	- modified for uM-FPU V3
'   V2.0 - January 17, 2005
'   - modified for uM-FPU V2 and PICBASIC PRO compiler
'	V1.0 - January 8, 2005
'	- original version
'=============================================================================

'-------------------- PIC oscillator speed ------------------------------------
define      OSC 20                      ' specify the speed of the oscillator
OSC_SPEED   con 20                      ' define PICBASIC constant

'-------------------- debug definitions ---------------------------------------
define      DEBUG_REG   PORTC
define      DEBUG_BIT   6
define      DEBUG_BAUD  19200
define      DEBUG_MODE  0

'-------------------- uM-FPU pin definitions ----------------------------------
FpuClk          var     PORTC.3			' SPI SCLK (uM-FPU SCLK)
FpuIn           var     PORTC.4			' SPI MISO (uM-FPU SOUT)
FpuOut          var     PORTC.5			' SPI MOSI (uM-FPU SIN)

include     "umfpuV3-spi.bas"           ' include uM-FPU V3.1 support routines

'-------------------- DS1620 pin definitions ---------------------------------

DS_RST      var     PORTA.0             ' DS1620 reset/enable
DS_CLK      var     PORTA.1             ' DS1620 clock
DS_DATA     var     PORTA.2             ' DS1620 data

LSBFIRST	con		0					' shiftout mode
LSBPRE		con		1					' shiftin mode

'-------------------- uM-FPU register definitions ----------------------------

DegC        con         1               ' degrees Celsius
DegF        con         2               ' degrees Fahrenheit
F1_8        con         3               ' constant 1.8

'-------------------- variables ----------------------------------------------

rawTemp     var         word            ' raw temperature reading

'=============================================================================
'-------------------- initialization -----------------------------------------
'=============================================================================

Reset:
  DEBUG 13, 10, 13, 10, "Demo 1: "

  GOSUB Fpu_Reset                       ' initialize uM-FPU
  IF fpu_status <> SYNC_CHAR THEN
    DEBUG "uM-FPU not detected."
    END
  ELSE
    GOSUB Print_Version                 ' display version string
  ENDIF
  DEBUG 13, 10, "---------------------"

  GOSUB Init_DS1620                     ' initialize DS1620

                                        ' load floating point constant
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, F1_8, ATOF, "1.8", 0, FSET0]

'=============================================================================
'-------------------- main routine -------------------------------------------
'=============================================================================

Main:
  GOSUB Read_DS1620                     ' get temperature reading from DS1620
  DEBUG 13, 10, 13, 10, "Raw Temp:  ", IHEX4 rawTemp

                                        ' send rawTemp to uM-FPU
                                        ' convert to floating point
                                        ' store in register
										' divide by 2 to get degrees Celsius
  SHIFTOUT FpuOut, FpuClk, MSBFIRST,_
	[SELECTA, DegC, LOADWORD, rawTemp\16, FSET0, FDIVI, 2]

                                        ' degF = degC * 1.8 + 32
  SHIFTOUT FpuOut, FpuClk, MSBFIRST,_
	[SELECTA, DegF, FSET, DegC, FMUL, F1_8, FADDI, 32]

  DEBUG 13, 10, "Degrees C: "    		' display degrees Celsius
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, DegC]
  format = 51
  GOSUB Print_FloatFormat

  DEBUG 13, 10, "Degrees F: "    		' display degrees Fahrenheit
  SHIFTOUT FpuOut, FpuClk, MSBFIRST, [SELECTA, DegF]
  format = 51
  GOSUB Print_FloatFormat

  PAUSE 2000                            ' delay, then get the next reading
  GOTO Main
  END

'-------------------- Init_DS1620 --------------------------------------------

Init_DS1620:
  ADCON1 = 6                            'configure A0-A3 for digital I/O
  LOW DS_RST                            'initialize pin states
  HIGH DS_CLK
  PAUSE 100

  HIGH DS_RST                           'configure for CPU control
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$0C, $02]
  LOW DS_RST
  PAUSE 100

  HIGH DS_RST                           'start temperature conversions
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$EE]
  LOW DS_RST
  PAUSE 1000                            'wait for first conversion
  RETURN

'-------------------- Read_DS1620 --------------------------------------------

Read_DS1620:
  HIGH DS_RST                           'read temperature value
  SHIFTOUT DS_DATA, DS_CLK, LSBFIRST, [$AA]
  SHIFTIN  DS_DATA, DS_CLK, LSBPRE, [rawTemp\9]
  LOW DS_RST                            'extend the sign bit
  IF rawTemp.BIT8 = 1 THEN rawTemp.HIGHBYTE = $FF
  RETURN
